@@ -24,49 +24,15 @@ def _hash_node(node: Any) -> int:
     return hash(node)
 
 
-def tree_hash(tree: PyTree) -> int:
-    hashed = jtu.tree_map(_hash_node, jtu.tree_leaves(tree))
-    return hash((*hashed, jtu.tree_structure(tree)))
+def tree_hash(*trees: PyTree) -> int:
+    hashed = jtu.tree_map(_hash_node, jtu.tree_leaves(trees))
+    return hash((*hashed, jtu.tree_structure(trees)))
 
 
-class ImmutableWrapper:
-    """Base class for all immutable wrappers that gets a special treatment inside `treeclass` wrapped classes.
-    In essence, this wrapper is rendered transparent inside `treeclass` wrapped classes
-    so that the wrapped value can be accessed directly, without the need to call `unwrap`.
-    This behavior is useful for myriads of use cases, such as freezing a value to avoid updating it
-    by `jax` transformations, or wrapping a value to make it hashable.
-
-    Example:
-        >>> import jax.tree_util as jut
-        >>> import pytreeclass as pytc
-        >>> @jtu.register_pytree_node_class
-        ... class TransparentWrapper(pytc.ImmutableWrapper):
-        ...    def __repr__(self):
-        ...        return f"TransparentWrapper({self.__wrapped__!r})"
-        ...    def tree_flatten(self):
-        ...        # return the unwrapped value as a tuple
-        ...        return (self.unwrap(),), None
-        ...    @classmethod
-        ...    def tree_unflatten(cls, _, xs):
-        ...        return TransparentWrapper(xs[0])
-
-        >>> @pytc.treeclass
-        ... class Tree:
-        ...    a:int = TransparentWrapper(1)
-        >>> tree = Tree()
-        >>> # no need to unwrap the value when accessing it
-        >>> assert type(tree.a)  is int
-        >>> print(tree)
-        Tree(a=TransparentWrapper(1))
-        >>> jtu.tree_leaves(tree)
-        [1]
-        >>> jtu.tree_leaves(tree, is_leaf=lambda x: isinstance(x, TransparentWrapper))
-        [TransparentWrapper(1)]
-    """
-
+class _ImmutableWrapper:
     def __init__(self, x: Any) -> None:
         # disable composition of Wrappers
-        vars(self)[_WRAPPED] = x.unwrap() if isinstance(x, ImmutableWrapper) else x
+        vars(self)[_WRAPPED] = x.unwrap() if isinstance(x, _ImmutableWrapper) else x
 
     def unwrap(self) -> Any:
         return getattr(self, _WRAPPED)
@@ -78,7 +44,7 @@ class ImmutableWrapper:
         raise AttributeError("Cannot delete from frozen instance.")
 
 
-class _HashableWrapper(ImmutableWrapper):
+class _HashableWrapper(_ImmutableWrapper):
     # used to wrap metadata to make it hashable
     # this is intended to wrap frozen values to avoid error when comparing
     # the metadata.
@@ -91,12 +57,12 @@ class _HashableWrapper(ImmutableWrapper):
         return tree_hash(self.unwrap())
 
 
-class FrozenWrapper(ImmutableWrapper):
+class _FrozenWrapper(_ImmutableWrapper):
     def __repr__(self):
         return f"#{self.unwrap()!r}"
 
     def __eq__(self, rhs: Any) -> bool:
-        if not isinstance(rhs, FrozenWrapper):
+        if not isinstance(rhs, _FrozenWrapper):
             return False
         return self.unwrap() == rhs.unwrap()
 
@@ -104,20 +70,18 @@ class FrozenWrapper(ImmutableWrapper):
         return tree_hash(self.unwrap())
 
 
-def _frozen_flatten(tree: Any) -> tuple[tuple, Any]:
+def _flatten(tree: Any) -> tuple[tuple, Any]:
     return (None,), _HashableWrapper(tree.unwrap())
 
 
-def _frozen_unflatten(treedef: Any, _: Sequence[Any]) -> PyTree:
-    tree = object.__new__(FrozenWrapper)  # type: ignore
-    vars(tree)[_WRAPPED] = treedef.unwrap()
-    return tree
+def _unflatten(treedef: Any, _: Sequence[Any]) -> PyTree:
+    return _FrozenWrapper(treedef.unwrap())
 
 
-jtu.register_pytree_node(FrozenWrapper, _frozen_flatten, _frozen_unflatten)
+jtu.register_pytree_node(_FrozenWrapper, _flatten, _unflatten)
 
 
-def freeze(wrapped: Any) -> FrozenWrapper:
+def freeze(wrapped: Any) -> _FrozenWrapper:
     r"""Freeze a value to avoid updating it by `jax` transformations.
 
     Example:
@@ -139,11 +103,12 @@ def freeze(wrapped: Any) -> FrozenWrapper:
         >>> jtu.tree_map(lambda x:x+100, a)
         [101, #2, 103]
 
-        >>> @pytc.treeclass
-        ... class Test:
+        >>> class Test(pytc.TreeClass):
         ...     a: float
         ...     @jax.value_and_grad
         ...     def __call__(self, x):
+        ...         # unfreeze `a` to update it
+        ...         self = jax.tree_util.tree_map(pytc.unfreeze, self, is_leaf=pytc.is_frozen)
         ...         return x ** self.a
 
         >>> # without `freeze` wrapping `a`, `a` will be updated
@@ -166,7 +131,7 @@ def freeze(wrapped: Any) -> FrozenWrapper:
         value: 4.0
         grad: Test(a=#2.0)
     """
-    return FrozenWrapper(wrapped)
+    return _FrozenWrapper(wrapped)
 
 
 def unfreeze(x: Any) -> Any:
@@ -180,19 +145,18 @@ def unfreeze(x: Any) -> Any:
         >>> frozen_value = pytc.freeze(1)
         >>> pytc.unfreeze(frozen_value)
         1
-
         >>> # usage with `jax.tree_map`
         >>> frozen_tree = jtu.tree_map(pytc.freeze, {"a": 1, "b": 2})
         >>> unfrozen_tree = jtu.tree_map(pytc.unfreeze, frozen_tree, is_leaf=pytc.is_frozen)
         >>> unfrozen_tree
         {'a': 1, 'b': 2}
     """
-    return x.unwrap() if isinstance(x, FrozenWrapper) else x
+    return x.unwrap() if isinstance(x, _FrozenWrapper) else x
 
 
 def is_frozen(wrapped: Any) -> bool:
     """Returns True if the value is a frozen wrapper."""
-    return isinstance(wrapped, FrozenWrapper)
+    return isinstance(wrapped, _FrozenWrapper)
 
 
 def is_nondiff(x: Any) -> bool:
