@@ -7,7 +7,7 @@ import math
 from collections import defaultdict
 from itertools import chain
 from types import FunctionType
-from typing import Any, Callable, Literal, Sequence
+from typing import Any, Callable, Literal
 
 import jax
 import numpy as np
@@ -52,7 +52,7 @@ def _node_pprint(
         return _dict_pprint(node, indent, kind, width, depth)
     if dc.is_dataclass(node):
         return _dataclass_pprint(node, indent, kind, width, depth)
-    if pytc.is_treeclass(node):
+    if isinstance(node, pytc.TreeClass):
         return _treeclass_pprint(node, indent, kind, width, depth)
     return _general_pprint(node, indent, kind, width, depth)
 
@@ -86,7 +86,6 @@ def _shape_dtype_pprint(
     shape = shape.replace("(", "[")
     shape = shape.replace(")", "]")
     shape = shape.replace(" ", ",")
-    shape = shape.replace("[]", "[0]")
     dtype = f"{node.dtype}".replace("int", "i")
     dtype = dtype.replace("float", "f")
     dtype = dtype.replace("complex", "c")
@@ -234,7 +233,8 @@ def _namedtuple_pprint(
         kvs = node._asdict().items()
         fmt = (f"{k}={_node_pprint(v,indent+1,kind,width,depth-1)}" for k, v in kvs)
         fmt = (", \n" + "\t" * (indent + 1)).join(fmt)
-    fmt = "namedtuple(\n" + "\t" * (indent + 1) + (fmt) + "\n" + "\t" * (indent) + ")"
+    name = type(node).__name__
+    fmt = f"{name}(\n" + "\t" * (indent + 1) + (fmt) + "\n" + "\t" * (indent) + ")"
     return _format_width(fmt, width)
 
 
@@ -250,7 +250,7 @@ def _dataclass_pprint(
     if depth == 0:
         fmt = "..."
     else:
-        kvs = ((F.name, vars(node)[F.name]) for F in dc.fields(node) if F.repr)
+        kvs = ((f.name, vars(node)[f.name]) for f in dc.fields(node) if f.repr)
         fmt = (f"{k}={_node_pprint(v,indent+1,kind,width,depth-1)}" for k, v in kvs)
         fmt = (", \n" + "\t" * (indent + 1)).join(fmt)
 
@@ -270,7 +270,7 @@ def _treeclass_pprint(
         fmt = "..."
 
     else:
-        kvs = ((F.name, vars(node)[F.name]) for F in pytc.fields(node) if F.repr)
+        kvs = ((k, vars(node)[k]) for k, f in node._fields.items() if f.repr)
         fmt = (f"{k}={_node_pprint(v,indent+1,kind,width,depth-1)}" for k, v in kvs)
         fmt = (", \n" + "\t" * (indent + 1)).join(fmt)
     fmt = f"{name}(\n" + "\t" * (indent + 1) + (fmt) + "\n" + "\t" * (indent) + ")"
@@ -352,12 +352,14 @@ def tree_str(
     return _node_pprint(tree, 0, "str", width, depth).expandtabs(tabwidth)
 
 
-def _resolve_names(names: Sequence[str], width: int) -> str:
+def _resolve_names(trace, width: int) -> str:
     # given a trace with a tuple of names, we resolve the names
     # to a single string
-    path, *paths = names
-    for name in paths:
-        path += "" if name.startswith("[") else "."
+    names, _, indices = trace
+    path = ""
+    for i, (name, index) in enumerate(zip(names, indices)):
+        name = f"[{index}]" if name is None else name
+        path += "" if name.startswith("[") else ("." if i > 0 else "")
         path += _node_pprint(name, 0, "str", width, float("inf"))
     return path
 
@@ -368,7 +370,7 @@ def _is_trace_leaf_depth_factory(depth: int):
     def is_trace_leaf(trace) -> bool:
         # trace is a tuple of (names, leaves, tracers, aux_data)
         # done like this to ensure 4-tuple unpacking
-        names, _, __, ___ = trace
+        names, _, __ = trace
         # stop tracing if depth is reached
         return False if depth is None else (depth <= len(names))
 
@@ -416,13 +418,15 @@ def tree_indent(
         is_leaf=is_leaf,
         is_trace_leaf=_is_trace_leaf_depth_factory(depth),
     ):
-        names, types = trace[0], trace[1]
+        names, types, indices = trace
 
-        for j, (name, type_) in enumerate(zip(names, types)):
-            if names[: j + 1] in seen:
-                continue  # skip printing the common parent node twice
-            seen.add(names[: j + 1])
+        for j, (name, type_, index) in enumerate(zip(names, types, indices)):
+            if (cur := (names[: j + 1], types[: j + 1], indices[: j + 1])) in seen:
+                # skip printing the common parent node twice
+                continue
+            seen.add(cur)
 
+            name = f"[{index}]" if name is None else name
             fmt += "\n" + "\t" * (j + 1)
             fmt += f"{_node_pprint(name,0,'str',width, depth )}"
             fmt += (
@@ -524,14 +528,12 @@ def tree_diagram(
 
     Example:
         >>> import pytreeclass as pytc
-        >>> @pytc.treeclass
-        ... class A:
+        >>> class A(pytc.TreeClass):
         ...        x: int = 10
         ...        y: int = (20,30)
         ...        z: int = 40
 
-        >>> @pytc.treeclass
-        ... class B:
+        >>> class B(pytc.TreeClass):
         ...     a: int = 10
         ...     b: tuple = (20,30, A())
 
@@ -868,12 +870,12 @@ def tree_summary(
         count = _calculate_count(leaf)
         COUNT += count
 
-        if trace == ((), (), (), ()):
+        if trace == ((), (), ()):
             # avoid printing the leaf trace (which is the root of the tree)
             # twice, once as a leaf and once as the root at the end
             continue
 
-        paths = _resolve_names(names=trace[0], width=60)
+        paths = _resolve_names(trace, width=60)
         types = _node_type_pprint(leaf, indent=0, kind="str", width=60, depth=depth)
         counts = f"{count:,}"
         ROWS += [[paths, types, counts]]
@@ -900,8 +902,7 @@ def tree_repr_with_trace(
 
     Example:
         >>> import pytreeclass as pytc
-        >>> @pytc.treeclass
-        ... class Test:
+        >>> class Test(pytc.TreeClass):
         ...    a:int = 1
         ...    b:float = 2.0
 
