@@ -1,4 +1,3 @@
-import functools as ft
 from typing import Any, Callable
 
 import jax
@@ -11,8 +10,7 @@ from pytreeclass._src.tree_freeze import _HashableWrapper, tree_hash
 
 
 def test_freeze_unfreeze():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class A:
+    class A(pytc.TreeClass, leafwise=True):
         a: int
         b: int
 
@@ -31,18 +29,15 @@ def test_freeze_unfreeze():
 
     assert pytc.unfreeze(pytc.freeze(1.0)) == 1.0
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class A:
+    class A(pytc.TreeClass, leafwise=True):
         a: int
         b: int
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class B:
+    class B(pytc.TreeClass, leafwise=True):
         c: int = 3
         d: A = A(1, 2)
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class A:
+    class A(pytc.TreeClass, leafwise=True):
         a: int
         b: int
 
@@ -59,16 +54,13 @@ def test_freeze_unfreeze():
     assert jtu.tree_leaves(b) == []
     assert jtu.tree_leaves(c) == [1, 2]
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l0:
+    class l0(pytc.TreeClass, leafwise=True):
         a: int = 0
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l1:
+    class l1(pytc.TreeClass, leafwise=True):
         b: l0 = l0()
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l2:
+    class l2(pytc.TreeClass, leafwise=True):
         c: l1 = l1()
 
     t = jtu.tree_map(pytc.freeze, l2())
@@ -77,13 +69,11 @@ def test_freeze_unfreeze():
     assert jtu.tree_leaves(t.c) == []
     assert jtu.tree_leaves(t.c.b) == []
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l1:
+    class l1(pytc.TreeClass, leafwise=True):
         def __init__(self):
             self.b = l0()
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l2:
+    class l2(pytc.TreeClass, leafwise=True):
         def __init__(self):
             self.c = l1()
 
@@ -96,8 +86,7 @@ def test_freeze_errors():
     class T:
         pass
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         a: Any
 
     t = Test(T())
@@ -112,18 +101,15 @@ def test_freeze_errors():
 
 
 def test_freeze_with_ops():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class A:
+    class A(pytc.TreeClass, leafwise=True):
         a: int
         b: int
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class B:
+    class B(pytc.TreeClass, leafwise=True):
         c: int = 3
         d: A = A(1, 2)
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: float = pytc.freeze(1.0)
         c: str = pytc.freeze("test")
@@ -143,8 +129,7 @@ def test_freeze_with_ops():
     jtu.tree_map(pytc.unfreeze, t, is_leaf=pytc.is_frozen)
     jtu.tree_map(pytc.freeze, t)
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         a: int
 
     t = jtu.tree_map(pytc.freeze, (Test(100)))
@@ -153,8 +138,7 @@ def test_freeze_with_ops():
     assert pytc.is_tree_equal(t.at[...].apply(lambda x: x + 1), t)
     assert pytc.is_tree_equal(t.at[...].reduce(jnp.sin, initializer=0), 0.0)
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         x: jnp.ndarray
 
         def __init__(self, x):
@@ -163,11 +147,9 @@ def test_freeze_with_ops():
     t = Test(jnp.array([1, 2, 3]))
     assert pytc.is_tree_equal(t.at[...].set(None), Test(x=None))
 
-    @ft.partial(pytc.treeclass, leafwise=True)
     class t0:
         a: int = 1
 
-    @ft.partial(pytc.treeclass, leafwise=True)
     class t1:
         a: int = t0()
 
@@ -175,13 +157,11 @@ def test_freeze_with_ops():
 
 
 def test_freeze_diagram():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class A:
+    class A(pytc.TreeClass, leafwise=True):
         a: int
         b: int
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class B:
+    class B(pytc.TreeClass, leafwise=True):
         c: int = 3
         d: A = A(1, 2)
 
@@ -193,8 +173,7 @@ def test_freeze_diagram():
 
 
 def test_freeze_mask():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: int = 2
         c: float = 3.0
@@ -205,8 +184,7 @@ def test_freeze_mask():
 
 
 def test_freeze_nondiff():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         a: int = pytc.freeze(1)
         b: str = "a"
 
@@ -220,8 +198,7 @@ def test_freeze_nondiff():
         .apply(pytc.unfreeze, is_leaf=pytc.is_frozen)
     ) == ["a"]
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class T0:
+    class T0(pytc.TreeClass, leafwise=True):
         a: Test = Test()
 
     t = T0()
@@ -234,21 +211,18 @@ def test_freeze_nondiff():
 
 
 def test_freeze_nondiff_with_mask():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class L0:
+    class L0(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: int = 2
         c: int = 3
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class L1:
+    class L1(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: int = 2
         c: int = 3
         d: L0 = L0()
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class L2:
+    class L2(pytc.TreeClass, leafwise=True):
         a: int = 10
         b: int = 20
         c: int = 30
@@ -266,13 +240,11 @@ def test_non_dataclass_input_to_freeze():
 
 
 def test_tree_freeze():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l0:
+    class l0(pytc.TreeClass, leafwise=True):
         x: int = 2
         y: int = 3
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l1:
+    class l1(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: l0 = l0()
 
@@ -293,13 +265,11 @@ def test_tree_freeze():
 
 
 def test_tree_unfreeze():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l0:
+    class l0(pytc.TreeClass, leafwise=True):
         x: int = 2
         y: int = 3
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l1:
+    class l1(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: l0 = l0()
 
@@ -324,13 +294,11 @@ def test_tree_unfreeze():
 
 
 def test_tree_freeze_unfreeze():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l0:
+    class l0(pytc.TreeClass, leafwise=True):
         x: int = 2
         y: int = 3
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class l1:
+    class l1(pytc.TreeClass, leafwise=True):
         a: int = 1
         b: l0 = l0()
 
@@ -347,8 +315,7 @@ def test_tree_freeze_unfreeze():
 
 
 def test_freeze_nondiff_func():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Test:
+    class Test(pytc.TreeClass, leafwise=True):
         a: int = 1.0
         b: int = 2
         c: int = 3
@@ -359,6 +326,7 @@ def test_freeze_nondiff_func():
 
     @jax.value_and_grad
     def loss_func(model):
+        model = model.at[...].apply(pytc.unfreeze, is_leaf=pytc.is_frozen)
         return jnp.mean((model(1.0) - 0.5) ** 2)
 
     @jax.jit
