@@ -1,5 +1,3 @@
-import functools as ft
-
 import jax
 import jax.tree_util as jtu
 import numpy.testing as npt
@@ -10,10 +8,9 @@ import pytreeclass as pytc
 
 
 def test_jit_freeze():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class Linear:
-        weight: jnp.ndarray
-        bias: jnp.ndarray
+    class Linear(pytc.TreeClass, leafwise=True):
+        weight: jax.Array
+        bias: jax.Array
         name: str
 
         def __init__(self, key, in_dim, out_dim):
@@ -26,8 +23,7 @@ def test_jit_freeze():
         def __call__(self, x):
             return x @ self.weight + self.bias
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class StackedLinear:
+    class StackedLinear(pytc.TreeClass, leafwise=True):
         l1: Linear
         l2: Linear
         l3: Linear
@@ -52,6 +48,7 @@ def test_jit_freeze():
 
     @jax.value_and_grad
     def loss_func(model, x, y):
+        model = model.at[...].apply(pytc.unfreeze, is_leaf=pytc.is_frozen)
         return jnp.mean((model(x) - y) ** 2)
 
     @jax.jit
@@ -106,18 +103,16 @@ def test_jit_freeze():
 
 
 def test_ops_with_jit():
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class T0:
-        a: int = 1
-        b: int = 2
-        c: int = 3
+    class T0(pytc.TreeClass, leafwise=True):
+        a: jax.Array = jnp.array(1)
+        b: jax.Array = jnp.array(2)
+        c: jax.Array = jnp.array(3)
 
-    @ft.partial(pytc.treeclass, leafwise=True)
-    class T1:
-        a: int = 1
-        b: int = 2
-        c: int = 3
-        d: jnp.ndarray = pytc.field(factory=lambda: jnp.array([1, 2, 3]))
+    class T1(pytc.TreeClass, leafwise=True):
+        a: jax.Array = jnp.array(1)
+        b: jax.Array = jnp.array(2)
+        c: jax.Array = jnp.array(3)
+        d: jax.Array = jnp.array([1, 2, 3])
 
     @jax.jit
     def getter(tree):
@@ -132,16 +127,18 @@ def test_ops_with_jit():
         return tree.at[...].apply(lambda _: 0)
 
     with pytest.raises(jax.errors.ConcretizationTypeError):
-        getter(T0())
+        pytc.is_tree_equal(getter(T0()), T0())
 
     assert pytc.is_tree_equal(T0(0, 0, 0), setter(T0()))
 
     assert pytc.is_tree_equal(T0(0, 0, 0), applier(T0()))
 
     with pytest.raises(jax.errors.ConcretizationTypeError):
-        getter(T1())
+        pytc.is_tree_equal(getter(T1()), T1())
 
-    assert pytc.is_tree_equal(T1(0, 0, 0, jnp.array([0, 0, 0])), setter(T1()))
+    assert pytc.is_tree_equal(
+        T1(jnp.array(0), jnp.array(0), jnp.array(0), jnp.array([0, 0, 0])), setter(T1())
+    )
 
     assert pytc.is_tree_equal(T1(0, 0, 0, jnp.array([0, 0, 0])), applier(T1()))
 
